/**
 *   nmr-common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */
package com.gestalt.jbi.nmr;

import org.w3c.dom.*;

import org.xml.sax.InputSource;

import java.io.StringReader;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;


/**
 * Utility class to create the jbi:message wrapper.  This is used to set content
 * on the normalized message.
 * Reference JBI spec section 5.5.1.1.4 WSDL 1.1-Defined Content Messages.
 */
public class NmrWrapperUtils {
    private static final String WRAPPER_DEFAULT_NAMESPACE_PREFIX = "tns";
    private static final String WRAPPER_DEFAULT_NAMESPACE = "http://java.sun.com/xml/ns/jbi/wsdl-11-wrapper";
    private static final String WRAPPER_MESSAGE = "jbi:message";
    private static final String WRAPPER_ATTRIBUTE_VERSION = "version";
    private static final String WRAPPER_ATTRIBUTE_VERSION_VALUE = "1.0";
    private static final String WRAPPER_ATTRIBUTE_TYPE = "type";
    private static final String WRAPPER_ATTRIBUTE_NAME = "name";
    private static final String WRAPPER_PART = "jbi:part";
    private DocumentBuilder docBuilder;
    private Document doc;

    public void init(QName type, List<String> attribs) {
        createNewMessage();

        Element msgWrapper = doc.createElementNS(WRAPPER_DEFAULT_NAMESPACE,
                WRAPPER_MESSAGE);
        msgWrapper.setAttribute(WRAPPER_ATTRIBUTE_VERSION,
            WRAPPER_ATTRIBUTE_VERSION_VALUE);

        String prefix = type.getPrefix();

        if ((prefix == null) || (prefix.length() == 0)) {
            prefix = WRAPPER_DEFAULT_NAMESPACE_PREFIX;
        }

        msgWrapper.setAttribute(WRAPPER_ATTRIBUTE_TYPE,
            prefix + ":" + type.getLocalPart());
        msgWrapper.setAttribute("xmlns:" + prefix, type.getNamespaceURI());

        if (null != attribs) {
            for (String name : attribs) {
                msgWrapper.setAttribute(WRAPPER_ATTRIBUTE_NAME, name);
            }
        }

        doc.appendChild(msgWrapper);
    }

    public void addPart(String partName, String part) {
        List<String> list = new ArrayList<String>();
        list.add(part);
        addParts(null, partName, list);
    }

    public void addComplexType(String xml)
        throws NmrMissingSchemaNamespaceException, Exception {
        Element content = null;

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        Document document = factory.newDocumentBuilder()
                                   .parse(new InputSource(new StringReader(xml)));
        content = document.getDocumentElement();

        addComplexType(content);
    }

    public void addComplexType(Element content)
        throws NmrMissingSchemaNamespaceException, Exception {
        Element root = doc.getDocumentElement();
        Element jbipart = doc.createElementNS(WRAPPER_DEFAULT_NAMESPACE,
                WRAPPER_PART);

        NamedNodeMap map = content.getAttributes();
        boolean namespaceFound = false;

        for (int i = 0; i < map.getLength(); i++) {
            Attr attribute = (Attr) map.item(i);

            if (attribute.getName().startsWith("xmlns")) {
                namespaceFound = true;

                break;
            }
        }

        if (!namespaceFound) {
            throw new NmrMissingSchemaNamespaceException(String.format(
                    "Complex type root element %s does not contain a schema namespace.",
                    content.getTagName()));
        }

        Node tempNode = doc.importNode(content, true);
        jbipart.appendChild(tempNode);
        root.appendChild(jbipart);
    }

    public void addEmptyPart() {
        Element root = doc.getDocumentElement();
        Element jbiElement = doc.createElementNS(WRAPPER_DEFAULT_NAMESPACE,
                WRAPPER_PART);
        root.appendChild(jbiElement);
    }

    public void addParts(String listName, String partName, List<String> parts) {
        Element root = doc.getDocumentElement();
        Element jbiElement = doc.createElementNS(WRAPPER_DEFAULT_NAMESPACE,
                WRAPPER_PART);
        root.appendChild(jbiElement);

        Element listElement = jbiElement;

        if (null != listName) {
            Element element = doc.createElement(listName);
            jbiElement.appendChild(element);
            listElement = element;
        }

        for (String part : parts) {
            if (null != partName) {
                Element element = doc.createElement(partName);
                element.setTextContent(part);
                listElement.appendChild(element);
            } else {
                listElement.setTextContent(part);
            }
        }
    }

    public Document getResult() {
        if (null != doc) {
            return doc;
        }
        return null;
    }

    private void createNewMessage() {
        try {
            if (docBuilder == null) {
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                docBuilder = factory.newDocumentBuilder();
            }

            doc = docBuilder.newDocument();
        } catch (ParserConfigurationException e) {
            //TODO do something clever with the exception
            e.printStackTrace();
        }
    }

    DocumentBuilder getDocBuilder() {
        return docBuilder;
    }

    void setDocBuilder(DocumentBuilder docBuilder) {
        this.docBuilder = docBuilder;
    }

    Document getDoc() {
        return doc;
    }

    void setDoc(Document doc) {
        this.doc = doc;
    }
}
